import type { RawEntry, ServerDefinition, ServerSource } from './config-schema.js';
export declare function normalizeServerEntry(name: string, raw: RawEntry, baseDir: string, source: ServerSource, sources: readonly ServerSource[]): ServerDefinition;
export declare const __configInternals: {
    ensureHttpAcceptHeader: typeof ensureHttpAcceptHeader;
};
declare function ensureHttpAcceptHeader(headers?: Record<string, string>): Record<string, string> | undefined;
export { ensureHttpAcceptHeader };
//# sourceMappingURL=config-normalize.d.ts.map