#!/usr/bin/env node
import { createRuntime } from './runtime.js';
export { parseCallArguments } from './cli/call-arguments.js';
export { handleCall } from './cli/call-command.js';
export { handleGenerateCli } from './cli/generate-cli-runner.js';
export { handleInspectCli } from './cli/inspect-cli-command.js';
export { extractListFlags, handleList } from './cli/list-command.js';
export { resolveCallTimeout } from './cli/timeouts.js';
export declare function runCli(argv: string[]): Promise<void>;
export declare function handleAuth(runtime: Awaited<ReturnType<typeof createRuntime>>, args: string[]): Promise<void>;
export declare function printAuthHelp(): void;
//# sourceMappingURL=cli.d.ts.map