interface InspectFlags {
    artifactPath: string;
    format: 'text' | 'json';
}
export declare function handleInspectCli(args: string[]): Promise<void>;
declare function parseInspectFlags(args: string[]): InspectFlags;
export declare const __test: {
    parseInspectFlags: typeof parseInspectFlags;
};
export {};
//# sourceMappingURL=inspect-cli-command.d.ts.map