export declare function resolveCommandArgument(value: string): string;
export declare function resolveCommandArguments(args: readonly string[]): string[];
export declare function normalizeTimeout(raw?: number): number | undefined;
export declare function raceWithTimeout<T>(promise: Promise<T>, timeoutMs: number): Promise<T>;
//# sourceMappingURL=utils.d.ts.map