import type { ConfigCliOptions } from './types.js';
export interface ListFlags {
    format: 'text' | 'json';
    source?: 'local' | 'import';
}
export declare function handleListCommand(options: ConfigCliOptions, args: string[]): Promise<void>;
//# sourceMappingURL=list.d.ts.map