import type { CommandInput } from './types.js';
export declare function inferNameFromCommand(command: CommandInput): string | undefined;
export declare function normalizeCommandInput(value: string): CommandInput;
export declare function looksLikeInlineCommand(value: string): boolean;
//# sourceMappingURL=name-utils.d.ts.map