export declare const supportsAnsiColor: boolean;
export declare function colorize(code: number, text: string): string;
export declare const dimText: (text: string) => string;
export declare const yellowText: (text: string) => string;
export declare const redText: (text: string) => string;
export declare const extraDimText: (text: string) => string;
export declare const cyanText: (text: string) => string;
export declare const boldText: (text: string) => string;
export declare const supportsSpinner: boolean;
//# sourceMappingURL=terminal.d.ts.map